/* 
* Rootfly:  A program for analyzing and measuring roots. 
* Copyright (c) 2007 Clemson University.
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or (at
* your option) any later version.
* 
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/ 

#include "ImageToWindowConverter.h"

ImageClientConverter::ImageClientConverter()
  : m_client_rect(0,0,0,0), m_image_size(0,0), m_window(0)
{
}

ImageClientConverter::~ImageClientConverter()
{
}

void ImageClientConverter::SetClientRect(const CRect& rect)
{
  m_client_rect = rect;
}

void ImageClientConverter::SetImageSize(const CSize& image_size)
{
  m_image_size = image_size;
}

void ImageClientConverter::SetWindow(int window)
{
  m_window = window;
}

CRect ImageClientConverter::GetClientRect(int window) const
{
  CRect r = m_client_rect;
  CPoint offset( 0, (window - m_window) * m_client_rect.Height() );
  r.OffsetRect( offset );
  return r;
}

CSize ImageClientConverter::GetImageSize() const
{
  return m_image_size;
}

int ImageClientConverter::GetWindow() const
{
  return m_window;
}

CRect ImageClientConverter::GetExtendedClientRect(int window1, int window2) const
{
  CRect r1 = GetClientRect(window1);
  CRect r2 = GetClientRect(window2);
  CRect r;
  r.UnionRect(r1, r2);
  return r;
}

CPoint ImageClientConverter::ClientToImage(const CPoint& point, int* window) const
{
  if (m_client_rect.Width() == 0 || m_client_rect.Height() == 0)  { *window = 0;  return CPoint(0, 0); } // needed b/c onmouse move calls this function before the class is initialized
  assert(m_client_rect.Width() > 0 && m_client_rect.Height() > 0);
  if (m_image_size.cy == 0)  { *window = 0;  return CPoint(0, 0); } // needed b/c onmouse move calls this function before the class is initialized
  assert(m_image_size.cy > 0);
//  if (p.x < m_client_rect.left )  p.x = m_client_rect.left;
//  if (p.x > m_client_rect.right)  p.x = m_client_rect.right;
  double scale_x = ((double) m_image_size.cx) / m_client_rect.Width();
  double scale_y = ((double) m_image_size.cy) / m_client_rect.Height();
  double ix = (point.x - m_client_rect.left) * scale_x;
  double iy = (point.y - m_client_rect.top ) * scale_y;
  int iix = (int) (ix + 0.5);
  int iiy = (int) (iy + 0.5);
  int w = iiy / m_image_size.cy;

  if (iiy < 0)  w--;

  *window = w + m_window;
  return CPoint( iix, iiy );
}

RPoint ImageClientConverter::ClientToImage(const CPoint& point) const
{
  int window;
  CPoint pt = ClientToImage( point, &window );
  return RPoint( pt.x, pt.y, window );
}

RPoint ImageClientConverter::ClientToImage(const RPoint& point) const
{
  CPoint pt( point.x, point.y );
  int window;
  pt = ClientToImage( pt, &window );
  return RPoint( pt.x, pt.y, window );
}

CPoint ImageClientConverter::ImageToClient(const CPoint& point, int window) const
{
  if (m_client_rect.Width() < 1 || m_client_rect.Height() < 1 )  return CPoint(0, 0); // needed b/c onmouse move calls this function before the class is initialized
  assert(m_client_rect.Width() > 1 && m_client_rect.Height() > 0);
  int left = m_client_rect.left;
  int top = m_client_rect.top + (window - m_window) * m_client_rect.Height();
  double scale_x = ((double) m_image_size.cx) / m_client_rect.Width();
  double scale_y = ((double) m_image_size.cy) / m_client_rect.Height();
  double ix = left + (point.x / scale_x);
  double iy = top  + (point.y / scale_y);
  int iix = (int) (ix + 0.5);
  int iiy = (int) (iy + 0.5);
  return CPoint( iix, iiy );
}

CPoint ImageClientConverter::ImageToClient(const RPoint& point) const
{
  return ImageToClient( CPoint(point.x, point.y), point.window );
}


