// RootflyGraphics.cpp: implementation of the RootflyGraphics class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "rootfly.h"
#include "RootflyGraphics.h"
#include "DataStructures.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


COLORREF RED =         RGB(255,0,0);
COLORREF GREEN =       RGB(0,255,0);
COLORREF BLUE =        RGB(0,0,255);
COLORREF BLACK =       RGB(0,0,0);
COLORREF DARK_BROWN =  RGB(139, 69, 19 );
COLORREF LIGHT_BROWN = RGB(205, 133, 63);
COLORREF WHITE =       RGB(255, 255, 255);
COLORREF SYS_GRAY =    ::GetSysColor(COLOR_3DFACE);

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////
RootflyGraphics::RootflyGraphics()
{

}

RootflyGraphics::~RootflyGraphics()
{

}

void RootflyGraphics::Initialize( const ImageClientConverter &icc, const Settings &s, const RootflyData &rfd )
{
  rfy_dlg_conv = &icc;
  rfy_dlg_settings = &s;
  rfy_dlg_data = &rfd;
}


// draws the diameter
// 'selected':  whether to draw root with 'selected' color
void RootflyGraphics::DrawDiameter(const RootInstance *inst, bool selected, CDC& dc) const
{
  COLORREF c;  
  if (selected)
  {
    c = inst->IsAlive() ? rfy_dlg_settings->GetRootDrawingColorSelected() :
    rfy_dlg_settings->GetRootDrawingColorDeadSelected();
  }
  else if (inst->IsAlive())
  {
    c = rfy_dlg_settings->GetRootDrawingColor();
  }
  else
  {
    c = rfy_dlg_settings->GetRootDrawingColorDead();
  }
  
  CRect image,rt,rb,rect;
  CRgn clip_rgn;
  const RDiameter *diam = inst->GetDiameter(0);
  CPoint conv_point1,conv_point2;
  double rad = diam->radius;
  conv_point1 = rfy_dlg_conv->ImageToClient( CPoint(diam->r_point.x,diam->r_point.y), diam->r_point.window );
  conv_point2 = rfy_dlg_conv->ImageToClient( CPoint(diam->r_point.x+(int)rad, diam->r_point.y+(int)rad), diam->r_point.window );
  
  rad = (int)((conv_point2.x-conv_point1.x + conv_point2.y-conv_point1.y)/2);
  rect = CRect(conv_point1.x - rad, conv_point1.y - rad, conv_point1.x + rad ,conv_point1.y + rad );
  
  CPen pen(PS_SOLID, 1, c);
  CPen* oldpen = dc.SelectObject(&pen);
  
  CBrush color;
  
  switch(inst->GetColor())
  {
  case RootInstance::COLOR_WHITE:       color.CreateSolidBrush( WHITE       ); break;
  case RootInstance::COLOR_LIGHTBROWN:  color.CreateSolidBrush( LIGHT_BROWN ); break;
  case RootInstance::COLOR_DARKBROWN:   color.CreateSolidBrush( DARK_BROWN  ); break;
  case RootInstance::COLOR_BLACK:       color.CreateSolidBrush( BLACK       ); break;
  default: 
    assert(0);  // root should have a color
    color.CreateSolidBrush( WHITE  ); 
    break;
  }
  dc.SelectObject(color); 
  
  dc.Ellipse( rect );
  dc.SelectObject(oldpen);  
}

// draws points on the selected root and diameter
void RootflyGraphics::DrawPoint(const RPoint* point, CDC& dc) const
{
  CRect rect;
  CBrush color;
  CPoint conv_point;
  int size = 6;
  
  conv_point = rfy_dlg_conv->ImageToClient( *point );
  
  rect = CRect(conv_point.x - size/2,conv_point.y - size/2,conv_point.x + size/2,conv_point.y + size/2);
  
  CPen pen(PS_SOLID, 1, BLACK);
  CPen* oldpen = dc.SelectObject(&pen);
  color.CreateSolidBrush(WHITE);
  dc.SelectObject(color);
  dc.Rectangle(rect);
  dc.SelectObject(oldpen);   
}

void RootflyGraphics::DrawLine( std::vector<RPoint> line, COLORREF c, CDC& dc ) const
{
  DrawLine(line, c, 1, dc);
}

void RootflyGraphics::DrawLine( std::vector<RPoint> line, COLORREF c, int width, CDC& dc ) const
{	 
  if ( line.size() >= 2 )
  {
    CRect rect,rt,rb;
    
    CRgn clip_rgn;    
    
    CPen pen(PS_SOLID, width, c );
    CPen* oldpen = dc.SelectObject(&pen);
    vector<RPoint>::const_iterator p1;
    vector<RPoint>::const_iterator p2;
    p1 = &(line.at(0));
    p2 = &(line.at(1));
    CPoint converted_p1, converted_p2;
    
    while( p2 != line.end())
    {
      converted_p1 = rfy_dlg_conv->ImageToClient( *p1 );
      converted_p2 = rfy_dlg_conv->ImageToClient( *p2 );
      dc.MoveTo(converted_p1.x, converted_p1.y);
      dc.LineTo(converted_p2.x, converted_p2.y);
      p1++;
      p2++;      
    } 
    
    dc.SelectObject(oldpen);
  }  
}

// Draws the selected root
void RootflyGraphics::DrawSelectedRoot(CDC& dc, int selected_id, int mode, CPoint mouse_point, bool xtend_tail) const
{
  CRect main;
  CPoint conv_point;
  vector<RPoint> extended_line;  
  const ApplicationData *app_data = rfy_dlg_data->GetApplicationData();
  const Experiment *exp = rfy_dlg_data->GetExperiment();
  const Tube *tube = exp->GetTube(app_data->GetTube());
  if (!tube)  return;
  const Root *root = tube->GetRoot( selected_id );
  if (!root)  return;
  const RootInstance *inst = root->GetRootInstance(app_data->GetSession());
  if (!inst)  return;
  const std::vector<RPoint> *line = inst->GetAllPoints();
  if (!line)  return;
  conv_point = rfy_dlg_conv->ImageToClient( line->at(0) );
  const RDiameter *diam = inst->GetDiameter(0);
  if (!diam)  return;
  
  DrawLine(*line, inst->IsAlive() ? rfy_dlg_settings->GetRootDrawingColorSelected() :
                                      rfy_dlg_settings->GetRootDrawingColorDeadSelected(), 2, dc);
  if (rfy_dlg_settings->RootIdIsEnabled())
  {
    CString temp;
    
    // displays the root id, checks that the current window is the same as the primary window for the RootInstance, if it is not
    // then it will show the RootInstance as "WindowID(PrimaryWindow
    if (app_data->GetWindow() != root->GetPrimaryWindow()) temp.Format("%d*",root->GetWindowID());
    else temp.Format("%d",root->GetWindowID());
    assert( line->size() > 0 );
    int window = line->size() > 0 ? line->at(0).window : app_data->GetWindow();
    main = rfy_dlg_conv->GetClientRect( window );
    if (conv_point.x-15 < main.left) conv_point.x +=30;
    if (conv_point.y-15 < main.top ) conv_point.y +=30;
    dc.SetTextColor(BLACK);
    CPoint topleft(conv_point.x-15, conv_point.y-15);
    dc.TextOut(topleft.x, topleft.y, temp);
    { // draw outline around root id
      CSize sz = dc.GetTextExtent(temp);
      CBrush brush(inst->IsAlive() ? rfy_dlg_settings->GetRootDrawingColorSelected() :
      rfy_dlg_settings->GetRootDrawingColorDeadSelected());
      CRect rect(topleft.x, topleft.y, topleft.x + sz.cx, topleft.y + sz.cy);
      rect.InflateRect(1, 1, 1, 1);
      dc.FrameRect( rect, &brush );
    }
  }
  if (rfy_dlg_settings->ShowDiametersIsEnabled())
  {    
    DrawDiameter(inst, true, dc);
  }
  if ( mode == MODE_ROOTSELECTED )
  {
    std::vector<RPoint>::const_iterator iter;  
    for ( iter = line->begin(); iter != line->end(); iter++ )
    { 
      DrawPoint(iter, dc);
    }
    if (rfy_dlg_settings->ShowDiametersIsEnabled())
    {
      const RPoint point = RPoint(diam->r_point.x, diam->r_point.y - diam->radius, diam->r_point.window);
      DrawPoint(&point, dc);
    }
  }  
  if ( mode == MODE_EXTENDINGENDPOINT )
  {
    RPoint rpt;
    int window;
    conv_point = rfy_dlg_conv->ClientToImage( mouse_point, &window );
    rpt.x = conv_point.x;
    rpt.y = conv_point.y;
    rpt.window = window;
    extended_line.push_back(rpt);
    if (xtend_tail)
    {
      rpt.x = line->back().x;
      rpt.y = line->back().y;
      rpt.window = line->back().window;
      extended_line.push_back(rpt);
    }
    else 
    {
      rpt.x = line->at(0).x;
      rpt.y = line->at(0).y;
      rpt.window = line->at(0).window;
      extended_line.push_back(rpt);
    }
    DrawLine(extended_line, inst->IsAlive() ? rfy_dlg_settings->GetRootDrawingColorSelected() :
                                              rfy_dlg_settings->GetRootDrawingColorDeadSelected(), 2, dc);
  }
}

// Draws all the roots in the data structure
void RootflyGraphics::DrawAllRoots(CDC& dc, int mode, bool zoomed) const
{
  if ( mode == MODE_NOFILELOADED ) return;  
  if ( mode == MODE_NOIMAGEDISPLAYED ) return;  
  const ApplicationData *app_data = rfy_dlg_data->GetApplicationData();
  const Experiment* exp = rfy_dlg_data->GetExperiment();   
  const Tube *tube = exp->GetTube(app_data->GetTube());
  if (tube == NULL) return;
  const vector<Root> *root = tube->GetAllRoots();
  if (root == NULL) return;
  CRect main; 
  CPoint conv_point;
  double rad; 
  CString str;  
  
  
  // Draws roots in zoom out mode
  if ( zoomed )
  {   
    for ( int k = 0; k < 3; k++ )
    {
      int window = app_data->GetWindow() + k - 1;
      if ( window < 1  ) continue;
      for ( int i = 0; i < root->size(); i++ )
      {
        if (root->at(i).GetPrimaryWindow() == window)
        {
          const RootInstance *inst = root->at(i).GetRootInstance(app_data->GetSession());
          if (inst == NULL)  continue;           
          if ( inst->IsAlive() || rfy_dlg_settings->DeadRootsIsEnabled() )
          {
            conv_point = rfy_dlg_conv->ImageToClient( *inst->GetPoint(0) );
            if ( ( mode == MODE_STARTINGMOVEALL || mode == MODE_MOVINGALLROOTS ) 
                && window == app_data->GetWindow())
            {
              DrawLine(*inst->GetAllPoints(), 
                        inst->IsAlive() 
                          ? rfy_dlg_settings->GetRootDrawingColorSelected() 
                          : rfy_dlg_settings->GetRootDrawingColorDeadSelected(), 
                        dc);
            }
            else
            {
              DrawLine(*inst->GetAllPoints(),
                        inst->IsAlive()
                          ? rfy_dlg_settings->GetRootDrawingColor()
                          : rfy_dlg_settings->GetRootDrawingColorDead(), 
                        dc);
            }
            if (rfy_dlg_settings->RootIdIsEnabled())
            {
              CString temp;
              
              // displays the root id, checks that the current window is the same as the primary window for the RootInstance, if it is not
              // then it will show the RootInstance as "WindowID(PrimaryWindow
              if (app_data->GetWindow() != root->at(i).GetPrimaryWindow()) temp.Format("%d*",root->at(i).GetWindowID());
              else temp.Format("%d",root->at(i).GetWindowID());
              main = rfy_dlg_conv->GetClientRect( window );
              if (conv_point.x-15 < main.left) conv_point.x +=30;
              if (conv_point.y-15 < main.top ) conv_point.y +=30;
              
              dc.SetTextColor(BLACK);
              dc.TextOut(conv_point.x-15,conv_point.y-15,temp);
            }
            if ( rfy_dlg_settings->ShowDiametersIsEnabled())
            {
              rad = (inst->GetDiameter(0))->radius;
              CRect bounds( inst->GetDiameter(0)->r_point.x - rad, 
                inst->GetDiameter(0)->r_point.y - rad, 
                inst->GetDiameter(0)->r_point.x + rad, 
                inst->GetDiameter(0)->r_point.y + rad);	                     
              
              //DrawCircle(bounds, inst->IsAlive()?m_set_data->GetRootDrawingColor():m_set_data->GetRootDrawingColorDead(), inst->GetColor());
              bool highlight = ( mode == MODE_MOVINGALLROOTS || mode == MODE_STARTINGMOVEALL ) && window == app_data->GetWindow();
              DrawDiameter(inst, highlight, dc);
            } 
          }
        }
      }
    }
  }
  
  // Not zoom out mode (i.e., standard mode)
  else
  {       
    for ( int i = 0; i < root->size(); i++ )
    {      
      const RootInstance *inst = root->at(i).GetRootInstance(app_data->GetSession());
      if (inst == NULL)  continue;         
      if ( inst->IsAlive() || rfy_dlg_settings->DeadRootsIsEnabled() )
      {
        conv_point = rfy_dlg_conv->ImageToClient( *inst->GetPoint(0) );
         
        if ( mode == MODE_STARTINGMOVEALL || mode == MODE_MOVINGALLROOTS)
        {
          DrawLine(*inst->GetAllPoints(),
                    inst->IsAlive() ? rfy_dlg_settings->GetRootDrawingColorSelected() 
                                    : rfy_dlg_settings->GetRootDrawingColorDeadSelected(),  dc);
        }
        else
        {
          DrawLine(*inst->GetAllPoints(),inst->IsAlive()?rfy_dlg_settings->GetRootDrawingColor():rfy_dlg_settings->GetRootDrawingColorDead(), dc);
        }
          
        if (rfy_dlg_settings->RootIdIsEnabled())
        {
          CString temp;
            
          // displays the root id, checks that the current window is the same as the primary window for the RootInstance, if it is not
          // then it will show the RootInstance as "WindowID(PrimaryWindow
          if (app_data->GetWindow() != root->at(i).GetPrimaryWindow()) temp.Format("%d(%d)",root->at(i).GetWindowID(),root->at(i).GetPrimaryWindow());
          else temp.Format("%d",root->at(i).GetWindowID());
          main = rfy_dlg_conv->GetClientRect( app_data->GetWindow() );
          if (conv_point.x-15 < main.left) conv_point.x +=30;
          if (conv_point.y-15 < main.top ) conv_point.y +=30;
            
          dc.SetTextColor(BLACK);
          dc.TextOut(conv_point.x-15,conv_point.y-15,temp);
        }
        if ( rfy_dlg_settings->ShowDiametersIsEnabled())
        {
          const RDiameter* dia = inst->GetDiameter(0);
          rad = dia->radius;
          CRect bounds( 
            dia->r_point.x - rad, 
            dia->r_point.y - rad, 
            dia->r_point.x + rad, 
            dia->r_point.y + rad);	                     
          bool highlight = ( mode == MODE_MOVINGALLROOTS || mode == MODE_STARTINGMOVEALL) && dia->r_point.window == app_data->GetWindow();
          DrawDiameter(inst, highlight, dc);
        }              
      }
    }
  }  
}

// Draws the new root that is currently being created
void RootflyGraphics::DrawCurrentRoot(CDC& dc, int mode, const RootInstance &ri, CPoint mouse_point) const
{
  const RDiameter *rdm;
  RPoint *temp;
  RPoint conv_point;
  double rad; 
  
  const ApplicationData *app_data = rfy_dlg_data->GetApplicationData();

  int window = app_data->GetWindow(); 
  if ( mode == MODE_DRAWINGNEWROOT )
  {	    	
    conv_point = rfy_dlg_conv->ClientToImage( mouse_point );
    // append current point to end of current root being drawn
    RootInstance temp_root = ri;
    temp = temp_root.CreatePoint();
    temp->x = conv_point.x;
    temp->y = conv_point.y;
    temp->window = conv_point.window;
    DrawLine(*temp_root.GetAllPoints(), rfy_dlg_settings->GetRootDrawingColorSelected(), dc);    
  }  
  
  //Draw the diameter that is being created
  if ( mode == MODE_SIZINGDIAMETER || mode == MODE_LOCATINGDIAMETER  )
  {
    DrawLine(*ri.GetAllPoints(), rfy_dlg_settings->GetRootDrawingColorSelected(), dc);
    rdm = ri.GetDiameter(0);
    rad = rdm->radius;	
    CRect bounds( rdm->r_point.x - rad, 
                  rdm->r_point.y - rad,
                  rdm->r_point.x + rad,
                  rdm->r_point.y + rad   );    
    DrawDiameter(&ri, true, dc);    
  }   
}

// Sets the clip region of the DC to either 
//    - the main image (in regular mode), or
//    - the union of the three images (in zoomed out mode).
// Should only need to call this function once before starting all the drawing.
// Note:  SelectClipRgn only seems to work for the real CDC, not for the MemDC.
void RootflyGraphics::SetClipRegion(CDC& dc, bool zoomed)
{
  CRgn clip_rgn;
  const ApplicationData *app_data = rfy_dlg_data->GetApplicationData();
  if ( zoomed )
  {
    CRect rect, rect1, rect2, rect3;
    int w = app_data->GetWindow();
    rect1 = rfy_dlg_conv->GetClientRect( w-1 );
    rect2 = rfy_dlg_conv->GetClientRect( w   );
    rect3 = rfy_dlg_conv->GetClientRect( w+1 );
    
    rect.UnionRect(rect1, rect2);
    rect.UnionRect(rect, rect3);
    clip_rgn.CreateRectRgnIndirect(&rect);
  }
  else
  {
    CRect rect;
    int w = app_data->GetWindow();
    rect = rfy_dlg_conv->GetClientRect( w );
    clip_rgn.CreateRectRgnIndirect(&rect);
  }
  //  { // clip region
  dc.SelectClipRgn(&clip_rgn); 
}

void RootflyGraphics::FillImageDisplayAreaWithGray(CDC& dc, CRect &rect)
{
  // fill image area with gray
  CBrush background(SYS_GRAY);
  dc.FillRect(rect, &background);
}

bool RootflyGraphics::RootIsInCurrentWindow( const RootInstance &inst ) const
{
  const vector<RPoint> *points = inst.GetAllPoints();
  const ApplicationData *app_data = rfy_dlg_data->GetApplicationData();
  for ( int i = 0; i < points->size() -1 ; i++ )
  {
    if ( points->at(i).window == app_data->GetWindow() )
      return true;
  }
  return false;
}